<?php
/*
    Plugin Name: Woocommerce - Sabpaisa Payment Gateway
	Plugin URI: https://sabpaisa.in/
    Description: Provides a Sabpaisa payment gateway for WooCommerce.
    Author: SRS Live Technologies Pvt. Ltd.
    Version: 1.0
	Author URI: https://sabpaisa.in/
*/

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly
// Check if WooCommerce is active
if (!in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) return;

require_once('classes/Authentication.php');
add_action('plugins_loaded', 'woocommerce_gateway_sabpaisa_init', 0);


function woocommerce_gateway_sabpaisa_init() {
	load_plugin_textdomain( 'wc_sabpaisa', false, dirname( plugin_basename( __FILE__ ) ) . '/languages' );
	
	if ( !class_exists( 'WC_Payment_Gateway' ) ) return;

	/**
 	 * Gateway class
 	 */
	class WC_Gateway_Sabpaisa extends WC_Payment_Gateway {

	     /**
         * Make __construct()
         **/	
		public function __construct(){
			
			$this->id 					= 'sabpaisa'; // ID for WC to associate the gateway values
			$this->method_title 		= 'Sabpaisa'; // Gateway Title as seen in Admin Dashboad
			$this->method_description	= 'Sabpaisa'; // Gateway Description as seen in Admin Dashboad
			$this->has_fields 			= false; // Inform WC if any fileds have to be displayed to the visitor in Frontend 
			
			$this->init_form_fields();	// defines your settings to WC
			$this->init_settings();		// loads the Gateway settings into variables for WC
						
			

			$this->title 			= $this->settings['title']; // Title as displayed on Frontend
			$this->description 		= $this->settings['description']; // Description as displayed on Frontend
			$this->clientcode = $this->settings['clientcode']; 
			$this->athenticationkey = $this->settings['athenticationkey']; 
			$this->athenticationiv = $this->settings['athenticationiv']; 
			$this->username = $this->settings['username']; 
			$this->password = $this->settings['password']; 
			$this->requestformat = $this->settings['requestformat']; 
					
            $this->msg['message']	= '';
            $this->msg['class'] 	= '';
			
			add_action('init', array(&$this, 'check_sabpaisa_response'));
            add_action('woocommerce_api_' . strtolower(get_class($this)), array($this, 'check_sabpaisa_response')); //update for woocommerce >2.0
			
			if ( version_compare( WOOCOMMERCE_VERSION, '2.0.0', '<' ) ) {
			add_action('woocommerce_update_options_payment_gateways', array(&$this, 'process_admin_options'));
		} else {
			add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
		}

           
            add_action('woocommerce_receipt_sabpaisa', array(&$this, 'receipt_page'));	
		} //END-__construct
		
        /**
         * Initiate Form Fields in the Admin Backend
         **/
		function init_form_fields(){

			$this->form_fields = array(
			'enabled' => array(
				'title' => __( 'Enable/Disable', 'wc_sabpaisa' ), 
				'label' => __( 'Enable Sabpaisa Payment Module', 'wc_sabpaisa' ), 
				'type' => 'checkbox', 
				'description' => '', 
				'default' => 'no'
				), 
			'title' => array(
				'title' => __( 'Title' ), 
				'type' => 'text', 
				'description' => __( 'This controls the title which the user sees during checkout.', 'wc_sabpaisa' ), 
				'default' => __( 'Sabpaisa', 'wc_sabpaisa' ),
				'css' => "width: 300px;"
				), 
			'description' => array(
				'title' => __( 'Description', 'wc_sabpaisa'), 
				'type' => 'textarea',
				'description' => __( 'This controls the description which the user sees during checkout.', 'wc_sabpaisa' ), 
				'default' => 'Pay securely via Sabpaisa.'
			),			
			'clientcode' => array(
				'title' => __( 'Client Code', 'wc_sabpaisa'), 
				'type' => 'text', 
				'description' => __( 'This is Client Code supplied by Sabpaisa.', 'wc_sabpaisa'), 
				'default' => ''
			),
			'username' => array(
				'title' => __( 'Username', 'wc_sabpaisa'), 
				'type' => 'text', 
				'description' => __( 'This is Username supplied by Sabpaisa.', 'wc_sabpaisa'), 
				'default' => ''
			), 
			'password' => array(
				'title' => __( 'Password', 'wc_sabpaisa'), 
				'type' => 'text', 
				'description' => __( 'This is Password supplied by Sabpaisa.', 'wc_sabpaisa'), 
				'default' => ''
			),
			'athenticationkey' => array(
				'title' => __( 'Authentication Key', 'wc_sabpaisa'), 
				'type' => 'text', 
				'description' => __( 'This is Authentication Key supplied by Sabpaisa.', 'wc_sabpaisa'), 
				'default' => ''
			),									
			'athenticationiv' => array(
				'title' => __( 'Authentication IV', 'wc_sabpaisa'), 
				'type' => 'text', 
				'description' => __( 'This is Authentication IV supplied by Sabpaisa.', 'wc_sabpaisa'), 
				'default' => ''
			),									
			'requestformat' => array(
				'title' => __( 'Request Format', 'wc_sabpaisa'), 
				'type' => 'select', 
				'options'=> array('encrypted' => 'Encrypted','non-encrypted' =>'Non Encrypted'),
				'description' => __( 'Fromat of request sent to Sabpaisa.', 'wc_sabpaisa'), 
				'default' => 'encrypted'
			)
		);		

		} //END-init_form_fields
		
      

        /**
         *  There are no payment fields, but we want to show the description if set.
         **/
		function payment_fields(){
			if( $this->description ) {
				echo wpautop( wptexturize( $this->description ) );
			}
		} //END-payment_fields
		
        /**
         * Receipt Page
         **/
		function receipt_page($order){
			echo '<p><strong>' . __('Thank you for your order.', 'wc_sabpaisa').'</strong><br/>' . __('The payment page will open soon.', 'wc_sabpaisa').'</p>';
			echo $this->generate_sabpaisa_form($order);
		} //END-receipt_page
    
        /**
         * Generate button link
         **/
		function generate_sabpaisa_form($order_id){
			global $woocommerce;
			$order = new WC_Order( $order_id );

			// Redirect URL
			if ( version_compare(WOOCOMMERCE_VERSION, '2.0.0', '>=' ) ) {
				$return_url = $this->get_return_url( $order );
			}
			else{$return_url = get_site_url() . "/";}

            		$productinfo = "Order $order_id";

			$txnid = $order_id.'_'.date("ymds");
			
			$return_url = strtok($return_url, '?');
			$spURL = null;
			$spDomain = "https://securepay.sabpaisa.in/SabPaisa/sabPaisaInit"; //URL provided by SabPaisa (Mandatory)
			$username = $this->username; //Username provided by Sabpaisa (Mandatory)
			$password = $this->password; //Password provided by Sabpaisa (Mandatory)
			$txnId = $txnid; //Transaction ID (Mandatory)
			$clientCode = $this->clientcode; //Client Code Provided by Sabpaisa (Mandatory)
			$authKey = $this->athenticationkey; //Authentication Key Provided By Sabpaisa
			$authIV = $this->athenticationiv; //Authentication IV Provided by Sabpaisa
			$requestformat = $this->requestformat; //request format for sabpaisa
			$txnAmt = $order->order_total;//Transaction Amount (Mandatory)
			$URLsuccess = $return_url; //Return URL upon successful transaction (Optional)
			$URLfailure = strtok($order->get_cancel_order_url(),'?'); //Return URL upon failed Transaction (Optional)
			$payerFirstName = $order->billing_first_name;	 //Payer's First Name (Optional)
			$payerLastName = $order->billing_last_name; //Payer's Last Name (Optional)
			$payerContact = substr( $order->billing_phone, -10 ); //Payer's Contact Number (Optional)
			$payerEmail = $order->billing_email;//Payer's Email Address (Optional)
			$payerAddress = $order->billing_address_1." ".$order->billing_address_2.", ".$order->billing_city; //Payer's Address (Optional)
			
			 $spURL = "?clientName=".$clientCode."&usern=".$username."&pass=".$password."&amountType=INR&channelId=&amt=".$txnAmt."&txnId=".$txnId."&firstName=".$payerFirstName."&lstName=".$payerLastName."&grNumber=&contactNo=".$payerContact."&Email=".$payerEmail."&Add=".$payerAddress."&midName&param1&param2&param3&param4&programId=&semstudentUin=&udf10&udf11&udf12&udf13&udf14&udf15&udf16&udf17&udf18&udf19&udf20&udf5&udf6&udf7&udf8&udf9&ru=".$URLsuccess."&failureURL=".$URLfailure;


			if($requestformat=='encrypted'){
			$EncryptDecrypt = new SabpaisaEncryptDecrypt();
			if($authIV != ''  && $authKey !=''){
				$spURL = $EncryptDecrypt -> encrypt($authKey,$authIV,$spURL);
				}
				$spURL = str_replace("+", "%2B",$spURL);
				$spURL="?query=".$spURL."&clientName=".$clientCode;
				}
 				$spURL = $spDomain.$spURL; 
				$spURL = str_replace(" ", "%20",$spURL);

			return '	<form action="'.$spURL.'" method="post" id="sabpaisa_payment_form">
				<input type="submit" class="button-alt" id="submit_sabpaisa_payment_form" value="'.__('Pay via Sabpaisa', 'wc_sabpaisa').'" /> <a class="button cancel" href="'.$order->get_cancel_order_url().'">'.__('Cancel order &amp; restore cart', 'wc_sabpaisa').'</a>
					<script type="text/javascript">
					jQuery(function(){
					jQuery("body").block({
						message: "'.__('Thank you for your order. We are now redirecting you to Payment Gateway to make payment.', 'wc_sabpaisa').'",
						overlayCSS: {
							background		: "#fff",
							opacity			: 0.6
						},
						css: {
							padding			: 20,
							textAlign		: "center",
							color			: "#555",
							border			: "3px solid #aaa",
							backgroundColor	: "#fff",
							cursor			: "wait",
							lineHeight		: "32px"
						}
					});
					jQuery("#submit_sabpaisa_payment_form").click();});
					</script>
				</form>';		
		} //END-generate_sabpais_form

        /**
         * Process the payment and return the result
         **/
        function process_payment($order_id){
			global $woocommerce;
            $order = new WC_Order($order_id);
			
			if ( version_compare( WOOCOMMERCE_VERSION, '2.1.0', '>=' ) ) { // For WC 2.1.0
			  	$checkout_payment_url = $order->get_checkout_payment_url( true );
			} else {
				$checkout_payment_url = get_permalink( get_option ( 'woocommerce_pay_page_id' ) );
			}

			return array(
				'result' => 'success', 
				'redirect'	=> $order->get_checkout_payment_url( true )
			);
		} //END-process_payment

        /**
         * Check for valid gateway server callback
         **/
        	function check_sabpaisa_response(){
            		global $woocommerce;					
        	} //END-check_sabpaisa_response
	} //END-class
	
	/**
 	* Add the Gateway to WooCommerce
 	**/
	function woocommerce_add_gateway_sabpaisa_gateway($methods) {
		$methods[] = 'WC_Gateway_Sabpaisa';
		return $methods;
	}//END-wc_add_gateway
	
	add_filter('woocommerce_payment_gateways', 'woocommerce_add_gateway_sabpaisa_gateway' );
	
} //END-init

/**
* 'Settings' link on plugin page
**/
add_filter( 'plugin_action_links', 'sabpaisa_add_action_plugin', 10, 5 );
function sabpaisa_add_action_plugin( $actions, $plugin_file ) {
	static $plugin;

	if (!isset($plugin))
		$plugin = plugin_basename(__FILE__);
	if ($plugin == $plugin_file) {

			$settings = array('settings' => '<a href="admin.php?page=wc-settings&tab=checkout&section=sabpaisa">' . __('Settings') . '</a>');
		
    			$actions = array_merge($settings, $actions);
			
		}
		
		return $actions;
}//END-settings_add_action_link

add_action('init','check_sabpaisa_response_outside');
function check_sabpaisa_response_outside(){
	

	if(isset($_REQUEST['query']) && isset($_REQUEST['clientCode'])){
		 global $woocommerce;
		//print_r($_REQUEST);		
		$query=$_REQUEST['query'];
		$Sabpaisaclass = new WC_Gateway_Sabpaisa();
		 $authKey=$Sabpaisaclass->athenticationkey;
		 $authIV=$Sabpaisaclass->athenticationiv;
		 $requestformat = $Sabpaisaclass->requestformat;
		 $decText = null;
	 	if($requestformat=='encrypted'){
			$EncryptDecrypt = new SabpaisaEncryptDecrypt();
			$decText = $EncryptDecrypt->decrypt($authKey,$authIV,$query);}
		else{

			$decText = $query;
		}
	 $finalArray = array();
	$asArr = explode( '&', $decText );

	foreach( $asArr as $val ){
	  $tmp = explode( '=', $val );
	  $finalArray[ $tmp[0] ] = $tmp[1];
	}
//print_r($finalArray);
if($finalArray['spRespStatus']=='success'){
	
	$txnid = explode("_",$finalArray['clientTxnId']);
	$order_id = $txnid[0];
	if($order_id != ''){
		try{
			$msg['message'] = "Thank you for the order. However, the transaction of Amount =".$finalArray['amount']." has been ".$finalArray['spRespStatus'].".<br/> Merchant Transaction Id  =".$finalArray['clientTxnId']." <br/> SabPaisa Transaction Id =".$finalArray['SabPaisaTxId']."<br/> Payment Mode =".$finalArray['payMode']."<br/>Transction Date=".$finalArray['transDate']."";
			$msg['class'] = 'success';
			$order = new WC_Order( $order_id );
			$order->payment_complete();
			$order->sabpaisatxId = $finalArray['SabPaisaTxId'];
			$order->Amount = $finalArray['amount'];
			$order->clientTxnId = $finalArray['clientTxnId'];
			$order->firstName = $finalArray['firstName'];
			$order->payMode = $finalArray['payMode'];
			$order->email = $finalArray['email'];
			$order->mobileNo = $finalArray['mobileNo'];
			$order->spRespStatus = $finalArray['spRespStatus'];
			$order->transDate = $finalArray['transDate'];
			$order->reMsg = $finalArray['reMsg'];
			$woocommerce->cart->empty_cart();
		}catch(Exception $e){
                        // $errorOccurred = true;
                        $msg = "Error";
					}
		}
		$redirect_url = $Sabpaisaclass->get_return_url( $order );
				//wc_print_notices();
				
				wp_redirect( $redirect_url );
                exit;
}
else if($finalArray['spRespStatus']=='pending'){
	
	$txnid = explode("_",$finalArray['clientTxnId']);
	$order_id = $txnid[0];
	if($order_id != ''){
		try{
			$msg['message'] = "Thank you for the order. Right now your payment status is pending. We will keep you posted regarding the status of your order through email";
			$msg['class'] = 'notice';
			$order = new WC_Order( $order_id );
			$order->add_order_note('Sabpaisa payment status is pending.<br/>Sabpaisa Transaction ID: '.$finalArray['clientTxnId'].'');
			$order->update_status('on-hold');
			$woocommerce->cart->empty_cart();
		}catch(Exception $e){
                        // $errorOccurred = true;
                        $msg = "Error";
					}
		}
}
else if($finalArray['spRespStatus']=='failure'){
	
	$txnid = explode("_",$finalArray['clientTxnId']);
	$order_id = $txnid[0];
	if($order_id != ''){
		try{
			$msg['message'] = "Thank you for the order. However, the transaction of Amount =".$finalArray['amount']." has been ".$finalArray['spRespStatus'].".<br/> Merchant Transaction Id  =".$finalArray['clientTxnId']." <br/> SabPaisa Transaction Id =".$finalArray['SabPaisaTxId']."<br/> Payment Mode =".$finalArray['payMode']."<br/>Transction Date=".$finalArray['transDate']."";
			$msg['class'] = 'error';
			$order = new WC_Order( $order_id );
			$order->sabpaisatxId = $finalArray['SabPaisaTxId'];
			$order->Amount = $finalArray['amount'];
			$order->clientTxnId = $finalArray['clientTxnId'];
			$order->firstName = $finalArray['firstName'];
			$order->payMode = $finalArray['payMode'];
			$order->email = $finalArray['email'];
			$order->mobileNo = $finalArray['mobileNo'];
			$order->spRespStatus = $finalArray['spRespStatus'];
			$order->transDate = $finalArray['transDate'];
			$order->reMsg = $finalArray['reMsg'];
		}catch(Exception $e){
                        // $errorOccurred = true;
                        $msg = "Error";
					}
		}
}


if ( function_exists( 'wc_add_notice' ) ) {
					wc_add_notice( $msg['message'], $msg['class'] );

				} else {
					if( 'success' == $msg['class'] ) {
						$woocommerce->add_message( $msg['message']);
					}else{
						$woocommerce->add_error( $msg['message'] );

					}
					$woocommerce->set_messages();
				}	
				
				
		}
}
